\ c2.fs -- C2 Protocol -- 10Aug13 rjn
\
] ( target forth)  here  ( start marker)
\ 
0 [if] \ ----------------------------[ FYI ]-----------------------------------
\
\ defined in prog-init.fs
\ 
:m c2ck    [ 0 .P0 ] m;
:m c2d     [ 1 .P0 ] m;
:m rs      [ 2 .P0 ] m;

-: c2d-in  $fd # P0MDOUT and!  [ c2d set ] ;
-: c2d-out $df # P0MDOUT ior!  ;
\ 
[then] \ ----------------------------------------------------------------------
\
\ 
\ -------------------------------[ c2 primitives ]-----------------------------
\ 
:m 80ns nop nop m;

\ The following are per AN127, rev. 1.2, pg. 11
:m c2-reset  \ 17Aug13 rjn
   c2ck-out [ c2ck clr ]  40 # us  [ c2ck set ]  5 # us m;  \ min. reset = 20 us

\ c2ck low time, tcl=20ns to 5 usec - AN127 recommends 80ns or more
\ c2d should be read at least 120 ns after the clock rising edge
\ c2d sampled on rising edge of c2ck
:m c2-clock  
\    c2ck-out 80ns [ c2ck clr ]  80ns  80ns  [ c2ck set ]  80ns 80ns m;
   ( c2ck-out 80ns) 80ns 80ns [ c2ck clr ]  80ns  80ns  [ c2ck set ]  m;  \ 17Aug13 rjn
:m c2-start  c2-clock m;  \ eye candy
:m c2-stop   c2-clock m;  \ eye candy

\ :m c2-start  c2d-in  [ c2ck clr ] 2 # us [ c2ck set ] c2d-out m;
\ :m c2-stop   c2d-in  [ c2ck clr ] 2 # us [ c2ck set ] m;

:m c2-send  ( n -)  \ send lsb first   
   c2ck-out  c2d-out
   8 # 7 #for
      [ rrc c2d movcb ]      \ send lsb from T to c2d
      c2-clock
   7 #next   drop m;

:m c2-rcv  ( - n)  \ receive lsb first 
   c2ck-out  c2d-in 0 #  
   8 # 7 #for
      c2-clock
      [ c2d movbc rrc ]      \ send lsb from c to T
   7 #next m;

:m c2-ins  ( n -)  \ 2-bit instruction write, send lsb first 
   c2ck-out  c2d-out  
   2 # 7 #for
      [ rrc c2d movcb ]      \ send lsb from T to c2d
      c2-clock
   7 #next   drop m;
\ 
\ --------------------------------[ timeout ]----------------------------------
\ 
\ need a timeout - otherwise, c2-wait, inbusy, outready will hang if no target
cpuHERE constant #to    1 cpuALLOT  \ # of times target found busy
cpuHERE constant #terr  1 cpuALLOT  \ # of timeout errors

50 constant #tomax  \ maximum timeout

:m /to  0 # #to #! m;
:m @to  #to #@ m;  \ do this after c2d-wait, inbusy, outready to get timing
: to?  ( - ?)   \ 0=no timeout, $ff=timout ( needs to be a colon def.) 
   [ #to inc ]  @to  #tomax # <if drop 0 # ; then drop $ff # ;
:m -to?  ( - ?)   to? $ff # xor m;  \ 0=timeout

:m /terr  0 # #terr #! m;  \ before lots of reads and writes
:m ?terr  @to #tomax # =if [ #terr inc ] then drop m;  \ after each status loop
:m @terr  #terr #@ m;  \ see how many errors in the operation

\ 
\ -------------------------[ read/write adr and data ]--------------------------
\
-: c2d?  0 # c2d if. $ff # swap drop then ;

\ : c2d-wait  c2d-in  begin c2-clock c2d until. ;  \ can hang
-: c2d-wait  \ wait with timeout
   c2d-in /to 0 # begin drop ( 1 # us) c2-clock c2d? to? ior until drop ;
\
\ ----- address write
\ : c2-awrite-ins  $03 # c2-ins ;
-: c2-awrite  ( n -)  \ write 8-bit address, lsb first, assumes c2d is input
   c2-start  $03 # c2-ins  c2-send  c2d-in  c2-stop ;
\
\ ----- address read
-: c2-aread  ( - n)   \ read status code from adr register 
   c2-start  $02 # c2-ins ( aread ins) c2-rcv  c2-stop ;
\
\ ----- wait for status (works)
\
\ loop takes a few microseconds ( each to tick is ~ 2 us)
-: ?outready \ wait for outready to set
   /to  0 # begin drop  c2-aread $01 # and  to? ior   until drop ?terr ; 
-: ?inbusy   \ wait for inbusy to clear 
   /to  0 # begin drop  c2-aread $02 # and -to? and 0=until drop ?terr ;
\ : ?outready  outready ?terr ;
\ : ?inbusy    inbusy   ?terr ;

\ ----- data write ok
-: c2-dwrite  ( .... len -)  \ write 8-bit data, lsb first
   c2-start  $01 # c2-ins  \ data write ins
   dup c2-ins  \ 2 bit length, 0 = 1 byte, 1 = 2 bytes, etc.  
   1+ 6 #for c2-send 6 #next  c2d-wait  c2-stop  ?inbusy ;
-: c2-dwrite1  ( n -)   0 # c2-dwrite ;

\ ----- data read
-: c2-dread  ( len - n1 ... nx)  \ read data
   c2-start  0 # c2-ins  \ data read ins
   dup c2-ins  \ 2 bit length, 0 = 1 byte, 1 = 2 bytes, etc.
   c2d-wait  1+ 6 #for c2-rcv 6 #next c2-stop ;
-: c2-dread1  ( - n)   0 # c2-dread ;

\ ----- send command
:m c2-cmd  ( cmd -)   \ send cmd to FPDAT
   $b4 # c2-awrite ( point to FPDAT)  c2-dwrite1 ( send cmd) 
   ?outready c2-dread1 drop ( wait for ACK -- $0d ok) m;

\ ----- init programming interface (PI)
-: /pi  \ init PI.  Only needs to be done once (e.g., by erase)
   c2-reset  $02 # ( FPCTL) c2-awrite \ point at FPCTL register
   $02 # c2-dwrite1  $04 # c2-dwrite1  $01 #  c2-dwrite1  30 # ms ; \ init seq.
\ 
\ ------------------------------[ chip erase ]---------------------------------
\ 
-: (e)  \ erase chip
   /pi                       \ init programming interface
   $03 # c2-cmd              \ device erase
   $de # c2-dwrite1  $ad # c2-dwrite1  $a5 # c2-dwrite1  \ write arming sequence
   c2-dread1 drop            \ read the response ($a5, last byte of arm seq.)
   200 # ms ; 

\ 
\ ------------------------[ read device and rev ID ]--------------------------
\ 
\ ----- read device ID for C8051F300 ( $04)
\ note: DID adr is $00, the reset default
-: did  ( - n)  c2-reset  c2-dread1 ; \ read device ID
\ 
\ ----- read rev. ID
\ note: chips at the shop all read $02
-: rid  ( - n)  c2-reset  $01 # ( REVID) c2-awrite  c2-dread1 ;  \ read rev ID


:m  set-tadr  ( -)  \ set the target address 
    high-adr #@ c2-dwrite1  \ write high adr
     low-adr #@ c2-dwrite1  \ write low adr 
m;

-: bump   [ low-adr inc ] low-adr #@ 0=if drop [ high-adr inc ] ; then drop ; 
\ 
\ -----------------------------[ block write ]---------------------------------
\ 
\ FPDAT valid for 30x, 31x, 32x, 326/7, 33x, 336/7, 35x, 36x, 37x/39x, 41x
\ see Table 10 in AN127 for other chips and FPDATA values
\
\ changed to use dnum vs. dlen -- 28Jul13 rjn
\ changed back to use dlen -- 26Aug13 rjn
-: block-write   \ length is in dlen, address in tadr
   ( /pi)   \ init before using series of block writes
   $07 # c2-cmd          \ block write 
   set-tadr              \ set target adr
   dlen #@ c2-dwrite1    \ send number to write
\   dbuf # a! dnum #@ 4 #for @+ c2-dwrite1 4 #next  \ write bytes
   0 ## p!  dlen #@ 4 #for @x+ c2-dwrite1 4 #next  \ write bytes  \ 26Aug13 rjn
   c2-dread1 drop ;      \ read response ($0D is ok, I get 64)
\ 
\ 
-: direct-write   \ length is in dlen, address in tadr
   ( /pi)
   $07 # c2-cmd          \ block write 
   set-tadr              \ set target adr
   dnum #@ c2-dwrite1    \ send number to write
   dnum #@ 4 #for key c2-dwrite1 4 #next  \ write bytes
   c2-dread1 drop   ;    \ read response ($0D is ok, I get 64)

\ 
0 [if] \ --------------------[ write test block]-------------------------------
\
-: block-write   \ test version -- writes 1, 2, ... from dbuf, starting at $80
   /pi
   $07 # c2-cmd          \ block write 
   set-tadr              \ set target adr
   #dbuf # c2-dwrite1    \ send length byte
   dbuf # a! #dbuf # 4 #for @+ c2-dwrite1 4 #next  \ write bytes
   c2-dread1 drop        \ read response ($0D is ok, I get 64)
;

-: wt$  string " error(s) writing test block (no target?)" 
-: wt  \ write test block (sequential data starting at $80)   
   fill-dbuf           \ fill dbuf with sequential data
   (e)                 \ erase chip
   /terr  block-write  \ write, accumulating errors, if any
   @terr 0=if drop ; then . wt$ ;
\ 
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ --------------------[ write one byte ]--------------------------------
\ 
\ Use clue to test write speed. ~ 262 us for one byte (no effort to tune)
\ Init PI before using
\ 
\ 
-: tbyte  ( n -)   \ writes one byte to target at current adr, increment adr
\   [ clue toggle ]
   $07 # c2-cmd      \ block write cmd 
   set-tadr          \ send start adr 
   1 # c2-dwrite1    \ length byte
       c2-dwrite1    \ write byte
\   [ low-adr inc ] low-adr #@ 0=if drop [ high-adr inc ] ; then drop 
   bump
\   [ clue toggle  clue set ]  \ takes about 262 usec (erase done first)
\   c2-dread1 drop        \ read response ($0D is ok, I get 64)
;
\ 
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ ---------------------[ test read ]------------------------------------
\
-: tb$  string " error(s) writing test byte" 
: tb  \ write test byte   
   (e)  /terr  $a5 # tbyte  \ write, accumulate errors, if any
   @terr 0=if drop ; then . tb$ ;
[then] \ ----------------------------------------------------------------------
\ 
\ ------------------------------[ block read ]---------------------------------
\ 
-: block-read   \ tadr contains Target adr, dnum is number to write
   ( /pi) 
   $06 # c2-cmd          \ block read 
   set-tadr              \ set target adr
   dnum #@ c2-dwrite1    \ send length byte
   ?outready  c2-dread1 drop  \ wait for data ready, read the response ($0D)
\   dnum #@ 4 #for ?outready c2-dread1 !dbuf 4 #next  \ read bytes
   0 ## p!  dnum #@ 4 #for ?outready c2-dread1 !x+ 4 #next  \ read bytes  26Aug13 rjn
;
 
-: direct-read   \ sends bytes out serial port as they are read
   ( /pi )  \ do this before a series of direct reads (e.g., f)
   $06 # c2-cmd          \ block read 
   set-tadr              \ set target adr
   dnum #@ c2-dwrite1    \ send length byte
   ?outready  c2-dread1 drop  \ wait for data ready, read the response ($0D)
   dnum #@  4 #for ?outready c2-dread1 (emit) 4 #next  \ read/send bytes
   dnum #@  dlen #! ;

0 [if] \ ---------------------[ test read ]------------------------------------
\ 
-: rb$  string " error(s) reading test block (no target?)" 
: rb  \ read test block (data starting at $80)   
   /dbuf                \ init dbuf with zeroes
   /terr  block-read    \ read, accumulating errors, if any
   @terr 0=if drop ; then . rb$ ; \ check for read errors
\ 
[then] \ ---------------------------------------------------------------------- 
\ 
0 [if] \ -----------------------[ byte read ]----------------------------------
\ 
\ note: you must do a /pi before reading bytes
-: rbyte  ( - n)   \ read one byte
\   /pi
   $06 # c2-cmd      \ block read 
   set-tadr          \ set target adr
   1 # c2-dwrite1    \ send length byte
   ?outready  c2-dread1 drop  \ wait for data ready, read the response ($0D)
   ?outready c2-dread1 \ read byte
   bump              \ increment adr
\   [ low-adr inc ] low-adr #@ 0=if drop [ high-adr inc ] ; then drop \ bump adr
;
\ 
[then] \ ---------------------------------------------------------------------- 
\
here ( stop marker) [ swap - ( -- n)
cs? [if] cr .( c2.fs              ) . .( bytes) [else] drop [then] 
]
\    
\ 
0 [if] ------------------------[ revisions ]-----------------------------------
\ 
Date	   By  Comment
=======	=== ===================================================================
10Aug13  rjn changed for conditional compilation
07Aug13  rjn eliminated unnecessary dictionary entries
31Jul13  rjn moved t, l, y commands to commands.fs
27Jun13  rjn started modifications for single-byte exchange protocol
17Mar13  rjn added byte read
15Mar13  rjn added delay in erase command
11Mar13  rjn fixed some primitives, added timeout & error indication
09Mar13  rjn all primitives and block-write from data buffer now works
26Feb13  rjn validated against detailed instructions in AN127
12Feb13  rjn original version
\
[then] \ ----------------------------------------------------------------------


