\ host-buffers.fs -- PROG establish/manage host buffers -- 10Aug13 rjn
\
] here
\
[ \ ==> gforth definitions
\
\ -----[ read chip.bin to buffer ]
\
32 1024 * constant #chip-buf  \ 32K is enough for 300 & 36x chips
create chip-buf  #chip-buf allot

variable cbuf>  \ current offset into chip.bin
: /cbuf>  0 cbuf> ! ;

: >cbuf  ( -- a)   chip-buf cbuf> @ + ;  \ convert to chip buffer adr
: @cbuf  ( -- n)   >cbuf c@ ;
: !cbuf  ( n --)   >cbuf c! ;

: @cbuf+  ( -- n)   @cbuf  1 cbuf> +! ;
: !cbuf+  ( n --)   !cbuf  1 cbuf> +! ;

: /cbuf  /cbuf>  #chip-buf 0 do 0 !cbuf+ loop  /cbuf> ;
: .cbuf>  cbuf> @ dup u. space [char] $ emit h. ;

0 value chip-fid   \ File ID.
0 value chip-#read \ number chars actually read

: read-chip  (  - )  /cbuf
   0 to chip-fid  s" ./target.bin" r/o open-file abort" Error opening target.bin"
     to chip-fid
   chip-buf #chip-buf chip-fid read-file abort" Error reading target.bin"
   to chip-#read  chip-fid close-file drop ;

\ : >hex   ( c - a l)   base @ >r hex  0 <# # # #> r> base ! ;

0 [if] \ -----[ test file ]
\
: tdata>cbuf  /cbuf  \ fill cbuf with test data
    target-size 0 do i $00ff and !cbuf+ loop ;

: create-test.bin  (  - )   tdata>cbuf
	0 to chip-fid   s" ./test.bin" delete-file drop
	s" ./test.bin" r/w create-file abort" Error creating test.bin"
	to chip-fid 
   chip-buf target-size chip-fid write-file abort" Error writing test.bin"
	chip-fid close-file abort" Error closing test.bin" ;

: read-test.bin  (  - )  /cbuf  \ read test file into chip.bin
   0 to chip-fid
   s" ./test.bin" r/o open-file  abort" Error opening test.bin"
   to chip-fid
   chip-buf #chip-buf chip-fid read-file abort" Error reading test.bin"
   to chip-#read  chip-fid close-file abort" Error closing test.bin" ;
\ 
[then] \ ----------------------------------------------------------------------
\ 
\ -----[dump chip.bin]
\ 

: .both  ( n --)  dup . [char] ( emit (dh.$) [char] ) emit  space ;

: .header   cr ." adr   00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F" ;
: ?show  ( c -- c')   dup $20 $7f within if emit else drop [char] . emit then ; 
: .16-cbytes  ( --)   cbuf> @  16 0 do @cbuf+ h. loop 
   space  cbuf> !  16 0 do @cbuf+ ?show loop ;
: ?range  ( a --)  chip-#read > if true abort" out of range!" then ;  
: .chip-size
   chip-#read 0> if    
      cr chip-#read  .both  ."  bytes in target.bin" 
   else
      cr ." target.bin read error (not in chip buffer)?
   then ;

: cdumpa  ( --)   \ dump 256 bytes at a time from chip-buf starting at cbuf>
\ does not affect cbuf>
   ." *** chip buffer dump ***" 
   cbuf> @
   chip-#read 0> if 
      .chip-size  ."  (any key for next, Esc to exit)"
      cr .header cr
      begin
         16 0 do cr cbuf> @ dup ?range dh. space .16-cbytes  loop  cr
      key 27 = until cbuf> !
   else drop drop ." read-chip first !"  then ;

: (cdump)  ( adr --)   cbuf> @  swap cbuf> ! cdumpa  cbuf> ! ;
: cdump  ( adr --)   depth 0= if 0 then (cdump) ;
: cdump-  ( --)   cbuf> @  dup 64 - dup 0< if drop 0 then cdump  cbuf> ! ;

\
\
\ -----[ image buffer ]
\
#chip-buf constant #image-buf
create image-buf  #image-buf allot

variable ibuf>  \ image buffer offset (pointer)
: /ibuf>  0 ibuf> ! ;

: >ibuf  ( -- a)   image-buf ibuf> @ + ;  \ convert to image adr
: @ibuf  ( -- n)   >ibuf c@ ;
: !ibuf  ( n --)   >ibuf c! ;

: @ibuf+  ( -- n)   @ibuf  1 ibuf> +! ;
: !ibuf+  ( n --)   !ibuf  1 ibuf> +! ;

: /ibuf  /ibuf>  #image-buf 0 do 0 !ibuf+ loop  /ibuf> ;
: .ibuf>  ibuf> @ dup u. space [char] $ emit h. ;

\
0 [if] \ -----------[dump image buffer, 64 bytes at a time]--------------------
\
1. The idump words do not affect the image buffer pointer, ibuf> .
2. idump- is normally used to verify the operation of a b and s sequence that
     sends data to the ipp download buffer (b) and receives a dump of data
     back from the ipp buffer, puting it in the image buffer (s).  After 
     this sequence, the image pointer, ibuf>, is advanced by 64 bytes.  The
     idump- command backs up #dbuf bytes before doing a dump.
3. Use idump> to specify a start address for the dump.
4. Use idump to dump #dbuf byte chunks, starting at the current ibuf> .
\
[then] \ ----------------------------------------------------------------------
\ 
: .16-ibytes  ( --)   ibuf> @  16 0 do @ibuf+ h. loop 
   space  ibuf> !  16 0 do @ibuf+ ?show loop ;
: idumpa  ( --)   \ dump 256 bytes at a time from image-buf at current ibuf>
\ note: does not affect ibuf>
   ." *** image buffer dump ***"
   ibuf> @
   .chip-size  ."  (any key for next, Esc to exit)"
   cr .header cr
   begin
      16 0 do cr ibuf> @ dup ?range dh. space .16-ibytes loop 
      cr
   key 27 = until ibuf> ! ;

: (idump)  ( adr --)  ibuf> @  swap ibuf> ! idumpa ibuf> ! ; 
: idump  ( adr --)   depth 0= if 0 then (idump) ;
: idump-  ( --)   ibuf> @  dup #dbuf - dup 0< if drop 0 then  idump  ibuf> ! ;
\ 
\ -----[ diff command: compare the two buffers ]
\ 
: .-  ( n --)    \ display number without trailing blank
   dup 10 < if [char] 0 emit then  s>d  0 d.r ; 
: .time  time&date drop drop drop .- [char] : emit  .-  [char] : emit  .- ;
: .fin  cr ." Finished: " .time ;

: diff   cr .time ."  Comparing target.bin to Target image: "  
   /ibuf>  /cbuf>  
   0 chip-#read  0 do 
      @ibuf+ @cbuf+ xor 0= 0= dup if ibuf> @ 1- space dh.$ then negate +
   loop 
   dup  cr .both ." errors" space space chip-#read swap - .both ." matches" ;
\ 
\ -----[ create target.img ]
\ 
0 value img-fid   
0 value img-#write \ number chars written

: save-image  (  --)  \ assumes target image in image buffer, ibuf
	0 to img-fid   s" ./target.img" delete-file drop
	s" ./target.img" r/w create-file abort" error creating target.img"
	to img-fid  chip-#read 0= abort" chip.bin not in chip buffer"
   image-buf chip-#read img-fid write-file abort" error writing target.img"
	img-fid close-file abort" error closing target.img" 
   cr ." image buffer saved to target.img" ;
\ 
] \ ===> myforth

here [ swap -  ( -- n)
cs? [if] cr .( host-buffers.fs      ) . .( bytes) [else] drop [then]
]
\
\
0 [if] ------------------------[ revisions ]-----------------------------------
\
Date	   By  Comment
=======	=== ===================================================================
10Aug13  rjn changed for conditional compilation summary
07Aug13  rjn added 0idump and 0cdump
26Jul13  rjn merged chip and image buffer files, cleaned up syntax
21Jul13  rjn original version  note ipp stands for interpose processor
\
[then] \ ----------------------------------------------------------------------


