\ timer2-timeout.fs -- timeout based on timer 2 interrupt -- 27Feb13 rjn
\
] here  \ ***** start marker
\
\ -----[ interrupt enable/disable ]
\ put in bootloader850.fs 30Sep13 rjn
\ 
:m |disable-interrupts  [ 7 .IE clr ] m;
:m |enable-interrupts   [ 7 .IE set ] m;
\ : enable-interrupts   |enable-interrupts ;
\ : disable-interrupts  |disable-interrupts ;

:m |t2-run   [ 2 .TMR2CN set ] m;
:m |t2-stop  [ 2 .TMR2CN clr ] m;

:m |t2-enable  [ 5 .IE set ] m;
:m |t2-disable [ 5 .IE clr ] m;

:m |t2-clear-overflow   [ 7 .TMR2CN clr ] m;
\ 
\ 
0 [if] \ --------------------[ timeout based on T2 ]---------------------------

1. /100us -- Sets a 100 us interrupt period.  The timeout is based on multiples 
   of a this period.  /100 us sets the interval at initialization and whenever 
   a new timeout is set (this ensures full intervals).
2. #to -- This cell holds the timeout, in multiples of 100 us.  The multiple 
   range is 1 to 255.
3. maximum timeout -- for a 100 us interval,  the maximum timeout is 
   (255)*(100 us) = 25.5 ms.  If larger values or finer increments are needed, 
   then the timeout interval can be changed or multiple timeouts
   can be calculated within the timeout loop.
4. !to -- sets the timeout interval (e.g., 5 # !to for a 500 us timeout).
   At each t2 interval, #to decrements until it reaches zero but
   it never go negative (e.g., from 0 to 255).  Thus, a timout is signalled 
   when #to equals 0.
5. /to -- terminates the timeout by forcing #to to zero.  Use this word
   to force loop termination.
6. #times -- Every interrupt, the #times cell is incremented unless #to has 
   decremented or been forced to zero.  Thus, when a loop is terminated by a 
   loop condition (i.e., with /to), #times contains the number of timeout 
   intervals that have expired.  This can be read to see how long it took 
   to satisfy the exit condition.  It can also be used as an indicator that a 
   timeout forced the exit, not a forced termination by the timed action.  
   If a full timeout has occurred, the timeout interval that was set will 
   match #times.
7. Typical usage: 
        \ termination condition in do-stuff forces #to to zero with /to
	1 # !to  0 # begin drop do-stuff  -to? 0=until drop
			- OR -
        \ timer-based delay of 1 ms
        :m 1ms  10 # !to  0 # begin drop -to? 0=until drop m;
                        - OR -
	\ see if loop exit was forced by do-stuff or by a timeout
	10 # dup !to  0 # begin drop do-stuff  -to? 0=until drop 
        @times xor 0=if drop ( timeout actions) ; then drop . . .
        
       
[then] \ ----------------------------------------------------------------------
\
cpuHERE constant #to    1 cpuALLOT  \ # of timeout intervals to do
cpuHERE constant #times 1 cpuALLOT  \ actual # of timeout intervals
\  
\ -----[ interval counting ]
\
\ 
: /100us   $fb # TMR2RLH #!  $33 # TMR2RLL #!  ;  \ 9.9980 kHz, ~ 100 us

:m -int  |disable-interrupts m;
:m +int  |enable-interrupts  m;

-: /to  0 # #to #! ;  \ forces a timeout 
-: !to  ( n - )  \ set new timeout interval, 1 to 255
   -int  /100us  #to #!  0 # #times #!  +int ;
-: -to?  ( - flag)  #to #@ ;  \ true (non-zero) if not timed out

-: @times  ( - n)   #times #@ ;  \ timeout intervals at termination

TIM2 interrupt : timer2-isr
   |t2-clear-overflow
\   [ t push  ]
\   [ clue toggle ]   \ 10 kHz reference out, see if t2 is working  
   -to? if [ #to dec ]  [ #times inc ] then drop 
\   [ t pop ]
   reti

-: /timer2  
   /100us
   $20 # CKCON  #!  \ set low byte clock source to SYSCLK
     0 # TMR2CN #!  \ 16-bit auto reload, clr overflow flag, T2 stopped
   |t2-enable  |t2-clear-overflow  |t2-run  |enable-interrupts 
;

0 [if] \ -----[ test ]
: tt  \ generate 1 kHz square wave on clue bit
	begin   
         	10 # !to  0 # begin drop -to? 0=until drop
		[ clue toggle ]
	again
[then] \ -------------

here [ swap -  cr .( timer2-timeout.fs   ) . .( bytes) ]

\ 
\ 
0 [if] \ -------------------[ Revision History ]-------------------------------

Date	By  Description
======= === ===================================================================
27Feb13 rjn initial version based on ELITE timer2 code
[then] \ ----------------------------------------------------------------------

